<?php declare(strict_types=1);

namespace Mille\PlatformConsorsFinanzRK\Bootstrap;

use Doctrine\DBAL\Connection;
use Shopware\Core\Checkout\Order\OrderDefinition;
use Shopware\Core\Checkout\Payment\PaymentMethodDefinition;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Shopware\Core\System\CustomField\CustomFieldTypes;
use Shopware\Core\System\User\UserDefinition;
use Symfony\Component\DependencyInjection\ContainerInterface;

class CustomFieldInstaller
{
    public const TEMPLATE = 'consors_finanz_rk_template';
    public const IS_CONSORS_FINANZ = 'consors_finanz_rk_payment';
    public const CONSORS_FINANZ_USER_IBAN = 'consors_finanz_rk_order_user_iban';
    public const CONSORS_FINANZ_USER_BIC = 'consors_finanz_rk_order_user_bic';
    public const CONSORS_FINANZ_IS_ORDER_BOOKED = 'consors_finanz_rk_is_order_booked';
    public const CONSORS_FINANZ_IS_ORDER_CANCELLED = 'consors_finanz_rk_is_order_cancelled';
    public const CONSORS_FINANZ_ORDER_CREDIT_NUMBERS = 'consors_finanz_rk_order_credit_numbers';
    public const CONSORS_FINANZ_CUSTOMER_NUMBER = 'consors_finanz_rk_customer_number';
    public const CONSORS_FINANZ_ORDER_INVOICE_NUMBER = 'consors_finanz_rk_order_invoice_number';
    public const CONSORS_FINANZ_ORDER_DATE = 'consors_finanz_rk_order_date';
    public const CONSORS_FINANZ_ORDER_DUE_DATE = 'consors_finanz_rk_order_due_date';
    public const CONSORS_FINANZ_ORDER_AMOUNT = 'consors_finanz_rk_order_amount';
    public const CONSORS_FINANZ_ORDER_INVOICE_TYPE = 'consors_finanz_rk_order_invoice_type';
    public const CONSORS_FINANZ_ORDER_INVOICE_REFERENCE = 'consors_finanz_rk_order_invoice_reference';
    public const CONSORS_FINANZ_ORDER_CHANGE_FLAG = 'consors_finanz_rk_order_change_flag';


    const ORDER_CUSTOMFIELD_SET = 'order_consors_finanz_rk_order_set';
    const USER_CUSTOMFIELD_SET = 'consors_finanz_rk_user_set';
    const PAYMENT_METHOD_CUSTOMFIELD_SET = 'consors_finanz_rk_payment_method_set';


    /** @var EntityRepositoryInterface */
    private $customFieldRepository;

    /** @var EntityRepositoryInterface */
    private $customFieldSetRepository;

    /**
     * @var ContainerInterface
     */
    private $container;
    /**
     * @var Connection
     */
    private $connection;

    public function __construct(ContainerInterface $container)
    {
        $this->customFieldSetRepository = $container->get('custom_field_set.repository');
        $this->customFieldRepository    = $container->get('custom_field.repository');
        $this->container                = $container;
        $this->connection               = $container->get(Connection::class);
    }

    public function install($context): void
    {
        foreach ($this->getCustomFieldSets() as $customFieldSet) {
            $criteria = new Criteria();
            $criteria->addFilter(new EqualsFilter('name', $customFieldSet['name']));

            $result = $this->customFieldSetRepository->search($criteria, $context->getContext());
            if (count($result->getIds()) === 0) {
                $this->customFieldSetRepository->upsert([$customFieldSet], $context->getContext());
            }
        }
    }

    public function uninstall($context): void
    {
        foreach ($this->getCustomFieldSets() as $customFieldSet) {
            foreach ($customFieldSet['customFields'] as $customField) {
                $criteria = new Criteria();
                $criteria->addFilter(new EqualsFilter('name', $customField['name']));

                $result = $this->customFieldRepository->search($criteria, $context->getContext());
                if (count($result->getIds()) > 0) {
                    foreach ($result->getIds() as $id) {
                        $this->customFieldRepository->delete([['id' => $id]], $context->getContext());
                    }
                }
            }

            $criteria2 = new Criteria();
            $criteria2->addFilter(new EqualsFilter('name', $customFieldSet['name']));

            $result2 = $this->customFieldSetRepository->search($criteria2, $context->getContext());
            if (count($result2->getIds()) > 0) {
                foreach ($result2->getIds() as $item) {
                    $this->customFieldSetRepository->delete([['id' => $item]], $context->getContext());
                }
            }
        }

        $this->deleteCustomFields();
    }

    public function activate($context)
    {
        $customFieldSets = $this->getCustomFieldSets();
        foreach ($customFieldSets as &$customFieldSet) {
            $customFieldSet['active'] = true;
            //$this->customFieldRepository->upsert([$customFieldSet], $context->getContext());
        }
    }

    public function deactivate($context)
    {
        $customFieldSets = $this->getCustomFieldSets();
        foreach ($customFieldSets as &$customFieldSet) {
            $customFieldSet['active'] = false;
            //$this->customFieldRepository->upsert([$customFieldSet], $context->getContext());
        }
    }

    /**
     * @return array
     */
    protected function getCustomFieldSets(): array
    {
        return [
            [
                'active' => true,
                'name' => self::ORDER_CUSTOMFIELD_SET,
                'config' => [
                    'label' => [
                        'de-DE' => 'Consors Finanz RK',
                        'en-GB' => 'Consors Finanz RK',
                    ],
                ],
                'customFields' => [
                    [
                        'active' => true,
                        'name' => self::CONSORS_FINANZ_IS_ORDER_BOOKED,
                        'type' => CustomFieldTypes::BOOL,
                    ],
                    [
                        'active' => true,
                        'name' => self::CONSORS_FINANZ_IS_ORDER_CANCELLED,
                        'type' => CustomFieldTypes::BOOL,
                    ],
                    [
                        'active' => true,
                        'name' => self::CONSORS_FINANZ_ORDER_CREDIT_NUMBERS,
                        'type' => CustomFieldTypes::TEXT,
                    ],
                    [
                        'active' => true,
                        'name' => self::CONSORS_FINANZ_CUSTOMER_NUMBER,
                        'type' => CustomFieldTypes::TEXT,
                        'config' => [
                            'customFieldPosition' => 1,
                            'label' => [
                                'de-DE' => 'Abweichende Kundennummer',
                                'en-GB' => 'Different customer number',
                            ],
                        ],
                    ],
                    [
                        'active' => true,
                        'name' => self::CONSORS_FINANZ_ORDER_INVOICE_NUMBER,
                        'type' => CustomFieldTypes::TEXT,
                        'config' => [
                            'customFieldPosition' => 2,
                            'label' => [
                                'de-DE' => 'Belegnummer',
                                'en-GB' => 'Invoice number',
                            ],
                        ],
                    ],
                    [
                        'active' => true,
                        'name' => self::CONSORS_FINANZ_ORDER_DATE,
                        'type' => CustomFieldTypes::TEXT,
                        'config' => [
                            'customFieldPosition' => 3,
                            'label' => [
                                'de-DE' => 'Datum',
                                'en-GB' => 'Date',
                            ],
                        ],
                    ],
                    [
                        'active' => true,
                        'name' => self::CONSORS_FINANZ_ORDER_DUE_DATE,
                        'type' => CustomFieldTypes::TEXT,
                        'config' => [
                            'customFieldPosition' => 4,
                            'label' => [
                                'de-DE' => 'Fälligkeitsdatum',
                                'en-GB' => 'Due date',
                            ],
                        ],
                    ],
                    [
                        'active' => true,
                        'name' => self::CONSORS_FINANZ_ORDER_AMOUNT,
                        'type' => CustomFieldTypes::TEXT,
                        'config' => [
                            'customFieldPosition' => 5,
                            'label' => [
                                'de-DE' => 'Betrag',
                                'en-GB' => 'Amount',
                            ],
                        ],
                    ],
                    [
                        'active' => true,
                        'name' => self::CONSORS_FINANZ_ORDER_INVOICE_TYPE,
                        'type' => CustomFieldTypes::TEXT,
                        'config' => [
                            'customFieldPosition' => 6,
                            'label' => [
                                'de-DE' => 'Belegtyp',
                                'en-GB' => 'Invoice type',
                            ],
                        ],
                    ],
                    [
                        'active' => true,
                        'name' => self::CONSORS_FINANZ_ORDER_INVOICE_REFERENCE,
                        'type' => CustomFieldTypes::TEXT,
                        'config' => [
                            'customFieldPosition' => 7,
                            'label' => [
                                'de-DE' => 'Belegreferenz',
                                'en-GB' => 'Invoice reference',
                            ],
                        ],
                    ],
                    [
                        'active' => true,
                        'name' => self::CONSORS_FINANZ_ORDER_CHANGE_FLAG,
                        'type' => CustomFieldTypes::BOOL,
                        'config' => [
                            'customFieldPosition' => 8,
                            'label' => [
                                'de-DE' => 'Changeflag',
                                'en-GB' => 'Changeflag',
                            ],
                        ],
                    ]
                ],
                'relations' => [
                    [
                        'entityName' =>  $this->container->get(OrderDefinition::class)->getEntityName()
                    ]
                ]
            ],
            [
                'active' => true,
                'name' => self::USER_CUSTOMFIELD_SET,
                'config' => [
                    'label' => [
                        'de-DE' => 'Consors Finanz RK',
                        'en-GB' => 'Consors Finanz RK',
                    ],
                ],
                'customFields' => [
                    [
                        'active' => true,
                        'name' => self::CONSORS_FINANZ_USER_IBAN,
                        'type' => CustomFieldTypes::TEXT,
                    ],
                    [
                        'active' => true,
                        'name' => self::CONSORS_FINANZ_USER_BIC,
                        'type' => CustomFieldTypes::TEXT,
                    ]
                ],
                'relations' => [
                    [
                        'entityName' =>  $this->container->get(UserDefinition::class)->getEntityName()
                    ]
                ]
            ],
            [
                'active' => true,
                'name' => self::PAYMENT_METHOD_CUSTOMFIELD_SET,
                'config' => [
                    'label' => [
                        'de-DE' => 'Consors Finanz RK',
                        'en-GB' => 'Consors Finanz RK',
                    ],
                ],
                'customFields' => [
                    [
                        'active' => true,
                        'name' => self::TEMPLATE,
                        'type' => CustomFieldTypes::TEXT,
                    ]
                ],
                'relations' => [
                    [
                        'entityName' =>  $this->container->get(PaymentMethodDefinition::class)->getEntityName()
                    ]
                ]
            ]
        ];
    }

    private function deleteCustomFields()
    {
        $this->connection->executeQuery("
            DELETE FROM `custom_field` WHERE `name` LIKE 'consors_finanz_rk_%'
        ");
    }
}
