<?php

declare(strict_types=1);

namespace Mille\PlatformConsorsFinanzRK\Bootstrap;

use Doctrine\DBAL\Connection;
use Mille\PlatformConsorsFinanzRK\MillePlatformConsorsFinanzRK;
use Shopware\Core\Framework\Api\Context\SystemSource;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;
use Mille\PlatformConsorsFinanzRK\PaymentMethod\PaymentMethodInterface;
use Shopware\Core\Checkout\Payment\PaymentMethodEntity;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\Plugin\Context\ActivateContext;
use Shopware\Core\Framework\Plugin\Context\DeactivateContext;
use Shopware\Core\Framework\Plugin\Context\InstallContext;
use Shopware\Core\Framework\Plugin\Context\UninstallContext;
use Shopware\Core\Framework\Plugin\Context\UpdateContext;
use Shopware\Core\Framework\Plugin\Util\PluginIdProvider;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Mille\PlatformConsorsFinanzRK\PaymentMethod\ConsorsFinanzDebit;
use Mille\PlatformConsorsFinanzRK\PaymentMethod\ConsorsFinanzInvoice;

class PaymentMethodInstaller
{
    public const PAYMENT_METHODS = [
        ConsorsFinanzInvoice::class,
        ConsorsFinanzDebit::class,
    ];

    public const AFTER_ORDER_PAYMENT_METHODS = [
        ConsorsFinanzInvoice::class,
        ConsorsFinanzDebit::class,
    ];

    /** @var PluginIdProvider */
    private $pluginIdProvider;

    /** @var EntityRepositoryInterface */
    private $paymentMethodRepository;

    /** @var EntityRepositoryInterface */
    private $salesChannelRepository;

    /** @var EntityRepositoryInterface */
    private $paymentMethodSalesChannelRepository;

    /** @var Connection */
    private $connection;

    public function __construct(ContainerInterface $container)
    {
        $this->pluginIdProvider                    = $container->get(PluginIdProvider::class);
        $this->paymentMethodRepository             = $container->get('payment_method.repository');
        $this->salesChannelRepository              = $container->get('sales_channel.repository');
        $this->paymentMethodSalesChannelRepository = $container->get('sales_channel_payment_method.repository');
        $this->connection                          = $container->get(Connection::class);
    }

    public function install(InstallContext $context): void
    {
        foreach ($this->getPaymentMethods() as $paymentMethod) {
            $this->upsertPaymentMethod($paymentMethod, $context->getContext());

            // Only do this within an install context otherwise this may
            // interfere badly with merchant configurations.
            $this->enablePaymentMethodForAllSalesChannels($paymentMethod, $context->getContext());
        }
    }

    public function update(UpdateContext $context): void
    {
        foreach ($this->getPaymentMethods() as $paymentMethod) {
            $this->upsertPaymentMethod($paymentMethod, $context->getContext());
        }
    }

    public function uninstall(UninstallContext $context): void
    {
        foreach ($this->getPaymentMethods() as $paymentMethod) {
            $this->deactivatePaymentMethod($paymentMethod, $context->getContext());
        }
    }

    public function activate(ActivateContext $context): void
    {
    }

    public function deactivate(DeactivateContext $context): void
    {
    }

    private function getPaymentMethods(): array
    {
        $paymentMethods = [];

        foreach (self::PAYMENT_METHODS as $paymentMethod) {
            $paymentMethods[] = new $paymentMethod();
        }

        return $paymentMethods;
    }

    private function upsertPaymentMethod(PaymentMethodInterface $paymentMethod, Context $context): void
    {
        $pluginId = $this->pluginIdProvider->getPluginIdByBaseClass(MillePlatformConsorsFinanzRK::class, $context);

        // Collect some common data which will be used for both update and insert
        $data = [
            'id'                => $paymentMethod->getId(),
            'handlerIdentifier' => $paymentMethod->getPaymentHandler(),
            'pluginId'          => $pluginId,
            'afterOrderEnabled' => in_array(get_class($paymentMethod), self::AFTER_ORDER_PAYMENT_METHODS),
            'customFields' => [
                CustomFieldInstaller::TEMPLATE  => $paymentMethod->getTemplate(),
                CustomFieldInstaller::IS_CONSORS_FINANZ => true,
            ],
        ];

        // Find existing payment method by ID for update / install decision
        $paymentMethodEntity = $this->findPaymentMethodEntity($paymentMethod->getId(), $context);

        // Decide whether to update an existing or install a new payment method
        if ($paymentMethodEntity instanceof PaymentMethodEntity) {
            $this->updatePaymentMethod($data, $context);
        } else {
            $this->installPaymentMethod($data, $paymentMethod, $context);
        }
    }

    private function installPaymentMethod(array $data, PaymentMethodInterface $paymentMethod, Context $context): void
    {
        $data = array_merge($data, [
            'name'         => $paymentMethod->getName(),
            'description'  => $paymentMethod->getDescription(),
            'position'     => $paymentMethod->getPosition(),
            'translations' => $paymentMethod->getTranslations(),
        ]);

        $this->paymentMethodRepository->create([$data], $context);
    }

    private function updatePaymentMethod(array $data, Context $context): void
    {
        $this->paymentMethodRepository->update([$data], $context);
    }

    private function enablePaymentMethodForAllSalesChannels(PaymentMethodInterface $paymentMethod, Context $context): void
    {
        $channels = $this->salesChannelRepository->searchIds(new Criteria(), $context);

        foreach ($channels->getIds() as $channel) {
            $data = [
                'salesChannelId'  => $channel,
                'paymentMethodId' => $this->getPaymentMethodId($paymentMethod->getPaymentHandler()),
            ];

            $this->paymentMethodSalesChannelRepository->upsert([$data], $context);
        }
    }

    private function deactivatePaymentMethod(PaymentMethodInterface $paymentMethod, Context $context): void
    {
        $data = [
            'id'     => $this->getPaymentMethodId($paymentMethod->getPaymentHandler()),
            'active' => false,
        ];

        $this->paymentMethodRepository->update([$data], $context);
    }

    private function getPaymentMethodId($paymentHandler): ?string
    {
        $context = new Context(new SystemSource());

        // Fetch ID for update
        $paymentCriteria = (new Criteria())->addFilter(new EqualsFilter('handlerIdentifier', $paymentHandler));
        $paymentIds = $this->paymentMethodRepository->searchIds($paymentCriteria, $context);

        if ($paymentIds->getTotal() === 0) {
            return null;
        }

        return $paymentIds->getIds()[0];
    }

    private function findPaymentMethodEntity(string $id, Context $context): ?PaymentMethodEntity
    {
        return $this->paymentMethodRepository
            ->search(new Criteria([$id]), $context)
            ->first();
    }
}
