<?php declare(strict_types=1);

namespace Mille\PlatformConsorsFinanzRK\Components\Consors\Request\Change;

use Mille\PlatformConsorsFinanzRK\Bootstrap\CustomFieldInstaller;
use Mille\PlatformConsorsFinanzRK\Components\SettingsTable;
use Mille\PlatformConsorsFinanzRK\PaymentMethod\ConsorsFinanzDebit;
use Shopware\Core\Checkout\Cart\LineItem\LineItem;
use Shopware\Core\Checkout\Customer\CustomerEntity;
use Shopware\Core\Checkout\Document\DocumentEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderAddress\OrderAddressEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderDelivery\OrderDeliveryEntity;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionCollection;
use Shopware\Core\Checkout\Order\Aggregate\OrderTransaction\OrderTransactionEntity;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Content\Product\ProductEntity;
use Shopware\Core\Framework\Api\Context\SystemSource;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;

class ChangeRequest
{
    /**
     * @var EntityRepositoryInterface
     */
    private $customerRepository;
    /**
     * @var EntityRepositoryInterface
     */
    private $documentRepository;
    /**
     * @var EntityRepositoryInterface
     */
    private $productRepository;

    public function __construct(
        EntityRepositoryInterface $customerRepository,
        EntityRepositoryInterface $documentRepository,
        EntityRepositoryInterface $productRepository
    ) {
        $this->customerRepository = $customerRepository;
        $this->documentRepository = $documentRepository;
        $this->productRepository = $productRepository;
    }

    public function getRequestParameters(DocumentEntity $document, bool $isGutschrift = false): array
    {
        $belege = $this->getBelege($document, $isGutschrift);
        $debitor = $this->getDebitor($document);

        return [
            'referenzen' => [
                'referenztyp' => 'bestellnummer',
                'referenzwert' => $document->getOrder()->getOrderNumber(),
            ],
            'debitor' => $debitor,
            'belege' => $belege,
        ];
    }

    private function getDebitor(DocumentEntity $document)
    {
        $order = $document->getOrder();
        $customer = $order->getOrderCustomer()->getCustomer();
        $orderBillingAddress = $order->getAddresses()->get($order->getBillingAddressId());

        return [
            'kundennummer' => $customer->getCustomerNumber(),
            'bezeichnung' => $customer->getFirstName() . ' ' . $customer->getLastName(),
            'person' => $this->getPerson($customer, $orderBillingAddress)
        ];
    }

    private function getBelege(DocumentEntity $document, $isGutschrift = false)
    {
        $order = $document->getOrder();
        $customer = $document->getOrder()->getOrderCustomer();
        $documentNumber = $document->getConfig()['documentNumber'];

        /** @var null|OrderTransactionCollection $transactions */
        $transactions = $order->getTransactions();

        /** @var OrderTransactionEntity $orderTransaction */
        $orderTransaction = $transactions->first();

        $paymentMethod = $orderTransaction->getPaymentMethod();

        $belegreferenz = (true == $isGutschrift) ? $this->getBelegReferenz($document) : $documentNumber;

        if ('' == $belegreferenz) {
            $belegreferenz = $documentNumber;
        }

        $beleg = [
            'kundennummer' => $customer->getCustomerNumber(),
            'belegtyp' => (true == $isGutschrift) ? SettingsTable::BELEG_TYPE_GUTSCHRIFT : SettingsTable::BELEG_TYPE_RECHNUNG,
            'belegnummer' => $documentNumber,
            'belegdatum' => $document->getCreatedAt()->format('Y-m-d'),
            'faellig' => date('Y-m-d', strtotime('+30 days', strtotime($document->getCreatedAt()->format('Y-m-d')))),
            'waehrung' => $order->getCurrency()->getIsoCode(),
            'betrag' => (true == $isGutschrift) ? $order->getAmountTotal() * -100 : $order->getAmountTotal() * 100,
            'betrag_netto' => (true == $isGutschrift) ? $order->getAmountNet() * 100 * -1 : $order->getAmountNet() * 100,
            'zahlart' => (ConsorsFinanzDebit::NAME == $paymentMethod->getName()) ? 'LA' : 'UEB',
            'rgbezahlt' => 'NEIN',
            'belegpositionen' => $this->getBelegPositions($order),
        ];

        if ($isGutschrift) {
            $beleg['belegreferenz'] = $belegreferenz;
            $beleg['belegtext'] = 'Gutschrift zu Rechnung ' . $belegreferenz . ' vom ' . $order->getCreatedAt()->format('d.m.Y');
        } else {
            $beleg['belegtext'] = 'Rechnung zur Bestellung ' . $order->getOrderNumber() . ' vom ' . $order->getCreatedAt()->format('d.m.Y');
        }

        return $beleg;
    }

    private function getBelegReferenz(DocumentEntity $document)
    {
        $criteria = new Criteria();
        $criteria->addAssociation('documentType');
        $criteria->addFilter(new EqualsFilter('documentType.technical_name', 'invoice'));
        $criteria->addFilter(new EqualsFilter('order_id', $document->getOrderId()));
        $context = new Context(new SystemSource());

        /** @var DocumentEntity $invoice */
        $invoice = $this->documentRepository->search($criteria, $context);

        if ($invoice) {
            return $invoice->getConfig()['documentNumber'];
        }

        return '';
    }

    protected function getBelegPositions(OrderEntity $order, $isManagementSystem = false)
    {
        $customer = $order->getOrderCustomer();
        $customFields = $order->getCustomFields();
        $lineItems = $order->getLineItems();

        $result = $articles = [];
        $key = 0;
        /** @var LineItem $lineItem */
        foreach ($lineItems as $lineItem) {

            /** @var ProductEntity $product */
            $product = $lineItem->getProduct();

            if (!$product) {
                continue;
            }

            $label = $product->get('translated')['name'];

            if ($label === null && $product->get('parentId') !== null) {
                $criteria = new Criteria();
                $criteria->addFilter(new EqualsFilter('id', $product->get('parentId')));
                $criteria->addAssociation('translations');

                /** @var ProductEntity $parentProduct */
                $parentProduct = $this->productRepository->search($criteria, new Context(new SystemSource()))->first();
                $label = $parentProduct->get('translated')['name'];
            }

            $taxRate = $lineItem->getPrice()->getTaxRules()->first()->getTaxRate();

            $result[] = [
                'kundennummer' => (true == $isManagementSystem && '' !== $customFields[CustomFieldInstaller::CONSORS_FINANZ_CUSTOMER_NUMBER]) ? $customFields[CustomFieldInstaller::CONSORS_FINANZ_CUSTOMER_NUMBER] : $customer->getCustomerNumber(),
                'belegpositionsnummer' => $key + 1,
                'betrag' => round($lineItem->getPrice()->getUnitPrice() * $lineItem->getQuantity() * 100, 2),
                'betrag_netto' => round($this->getNetto($lineItem->getPrice()->getUnitPrice(), $taxRate) * $lineItem->getQuantity() * 100, 2),
                'umsatzsteuersatz' => $taxRate * 100,
                'waehrung' => $order->getCurrency()->getIsoCode(),
                'artikel' => [
                    'menge' => $lineItem->getQuantity(),
                    'einheit' => '',
                    'artikelnummer' => $lineItem->getPayload()['productNumber'],
                    'artikelbezeichnung' => $label,
                    'artikelzusatz' => '',
                    'betrag' => round($lineItem->getPrice()->getUnitPrice() * 100, 2),
                    'betrag_netto' => round($this->getNetto($lineItem->getPrice()->getUnitPrice(), $taxRate) * 100, 2),
                    'umsatzsteuersatz' => $taxRate * 100,
                    'warengruppe' => '',
                ]
            ];

            $key++;
        }

        /** @var OrderDeliveryEntity $delivery */
        $delivery = $order->getDeliveries()->first();

        if (!empty($delivery)) {
            $taxRate = $delivery->getShippingCosts()->getCalculatedTaxes()->first()->getTaxRate();
             $result[$key] =  [
                 'kundennummer' => (true == $isManagementSystem && '' != $customFields[CustomFieldInstaller::CONSORS_FINANZ_CUSTOMER_NUMBER]) ? $customFields[CustomFieldInstaller::CONSORS_FINANZ_CUSTOMER_NUMBER] : $customer->getCustomerNumber(),
                    'belegpositionsnummer' => $key + 1,
                    'betrag' => round($order->getShippingTotal() * 100, 2),
                    'betrag_netto' => round($this->getNetto($order->getShippingTotal(), $taxRate) * 100, 2),
                    'umsatzsteuersatz' => round($taxRate * 100, 2),
                    'waehrung' => $order->getCurrency()->getIsoCode(),
                    'artikel' => [
                        'menge' => $delivery->getShippingCosts()->getQuantity(),
                        'einheit' => '',
                        'artikelnummer' => 'Versandkosten',
                        'artikelbezeichnung' => $delivery->getShippingMethod()->getName(),
                        'artikelzusatz' => '',
                        'betrag' => round($order->getShippingTotal() * 100, 2),
                        'betrag_netto' => round($this->getNetto($order->getShippingTotal(), $taxRate) * 100, 2),
                        'umsatzsteuersatz' => round($taxRate * 100, 2),
                        'warengruppe' => '',
                    ]
            ];
        }
        return $result;
    }

    protected function getNetto($brutto, $taxpercent)
    {
        $b = doubleval($brutto);
        $p = doubleval($taxpercent);
        if ($p <= 0) {
            return $b;
        }
        return (($b)/(1+($p/100)));
    }

    private function getArtikelZusatz(array $options = [])
    {
        if (empty($options)) {
            return '';
        }

        $result = '';
        foreach ($options as $option) {
            if ('' == $option['group']) {
                continue;
            }
            $result .= $option['group'] . ': ' . $option['option'] . ',';
        }

        return trim($result, ',');
    }

    protected function getPerson(CustomerEntity $customer, OrderAddressEntity $address, $liefer = false)
    {
        $firstName = ($address->getFirstName() != null) ? $address->getFirstName() : $customer->getFirstName();
        $lastName = ($address->getLastName() != null) ? $address->getLastName() : $customer->getLastName();

        $person = [
            'salutation' => $address->getSalutation()->getDisplayName(),
            'vorname' => ($liefer === true) ? $firstName : $customer->getFirstName(),
            'name' => ($liefer === true) ? $lastName : $customer->getLastName(),
            'geburtsdatum' => (is_string($customer->getBirthday())) ? $customer->getBirthday() : $customer->getBirthday()->format('Y-m-d'),
            'hauptanschrift' => [
                'adresse' => $this->getAddress($address),
                'zusatz' => $address->getAdditionalAddressLine1()
            ],
            'kontakte' => [
                [
                    'typ' => 'mobil',
                    'kontakt' => $address->getPhoneNumber()
                ],
                [
                    'typ' => 'email',
                    'kontakt' => $customer->getEmail()
                ]
            ],
            'rechtsform' => '',
            'handelsregisternummer' => '',
            'amtsgericht' => '',
        ];

        $bankData = $this->getUserBankData($customer);
        if (!empty($bankData)) {
            $person = array_merge(
                $person,
                ['bankverbindungen' => $bankData]
            );
        }
        return $person;
    }

    protected function getAddress(OrderAddressEntity $address)
    {
        return [
            'strasse' => $address->getStreet(),
            'plz' => $address->getZipcode(),
            'ort' => $address->getCity(),
            'land' => $address->getCountry()->getIso(),
        ];
    }

    protected function getUserBankData(CustomerEntity $customer)
    {
        $customFields = $customer->getCustomFields();

        $iban = $bic = '';
        if ($customFields) {
            $iban = $customFields[CustomFieldInstaller::CONSORS_FINANZ_USER_IBAN];
            $bic = $customFields[CustomFieldInstaller::CONSORS_FINANZ_USER_BIC];
        }

        return [
          'iban' => $iban,
          'bic' => $bic,
        ];
    }

    function dumb($data){
        highlight_string("<?php\n " . var_export($data, true) . "?>");
        echo '<script>document.getElementsByTagName("code")[0].getElementsByTagName("span")[1].remove() ;document.getElementsByTagName("code")[0].getElementsByTagName("span")[document.getElementsByTagName("code")[0].getElementsByTagName("span").length - 1].remove() ; </script>';
        die();
    }
}
