<?php declare(strict_types=1);

namespace Mille\PlatformConsorsFinanzRK\Components\Consors\Request\Customer;

use Mille\PlatformConsorsFinanzRK\Components\ConfigReader\ConfigReaderInterface;
use Mille\PlatformConsorsFinanzRK\Components\SettingsTable;
use Mille\PlatformConsorsFinanzRK\Struct\Configuration;
use RuntimeException;
use DateTime;
use Shopware\Core\Checkout\Customer\Aggregate\CustomerAddress\CustomerAddressEntity;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\Country\CountryEntity;
use Shopware\Core\System\Language\LanguageEntity;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Core\System\Salutation\SalutationEntity;
use Symfony\Component\HttpFoundation\RequestStack;

class CustomerRequest
{
    /** @var EntityRepositoryInterface */
    private $languageRepository;

    /** @var RequestStack */
    private $requestStack;

    /** @var EntityRepositoryInterface */
    private $salutationRepository;

    /** @var EntityRepositoryInterface */
    private $countryRepository;
    /**
     * @var ConfigReaderInterface
     */
    private $configReader;

    /** @var Configuration */
    protected $configuration;

    public function __construct(
        EntityRepositoryInterface $languageRepository,
        EntityRepositoryInterface $salutationRepository,
        EntityRepositoryInterface $countryRepository,
        RequestStack $requestStack,
        ConfigReaderInterface $configReader
    ) {
        $this->languageRepository   = $languageRepository;
        $this->salutationRepository = $salutationRepository;
        $this->countryRepository    = $countryRepository;
        $this->requestStack         = $requestStack;
        $this->configReader = $configReader;
    }

    public function getRequestParameters(RequestDataBag $dataBag, $anfrageart, SalesChannelContext $context): array
    {
        $this->configuration = $this->configReader->read($context->getSalesChannel()->getId());

        $customer = $context->getCustomer();

        /** @var RequestDataBag $consorsFinanzData */
        $consorsFinanzData = $dataBag->get('consorsfinanz');

        if (null === $customer) {
            throw new RuntimeException('missing customer');
        }

        $language = $this->getCustomerLanguage($context);

        if (null === $language->getLocale()) {
            throw new RuntimeException('missing language locale');
        }

        $billingAddress = $customer->getActiveBillingAddress();

        if (null === $billingAddress) {
            throw new RuntimeException('missing customer billing address');
        }

        /** @var CountryEntity $country */
        $country = $this->getCustomerCountry($billingAddress, $context->getContext())->getIso();

        $personalData = [
            'kontonummer' => $customer->getCustomerNumber(),
            'person' => [
                'vorname'            => $billingAddress->getFirstName(),
                'name'               => $billingAddress->getLastName(),
                'salutation'         => $this->getCustomerSalutation($billingAddress, $context->getContext())->getDisplayName(),
                'hauptanschrift'     => [
                    'adresse' => [
                        'strasse'     => $billingAddress->getStreet(),
                        'plz'         => $billingAddress->getZipcode(),
                        'ort'         => $billingAddress->getCity(),
                        'land'        => $country,
                    ]
                ],
                'kontakte' => [
                    [
                        'typ' => 'mobil',
                        'kontakt' => ($consorsFinanzData && $consorsFinanzData->get('phone')) ?$consorsFinanzData->get('phone') : $billingAddress->getPhoneNumber(),
                    ],
                    [
                        'typ' => 'email',
                        'kontakt' => $customer->getEmail()
                    ],
                ],
                'rechtsform' => '',
                'handelsregisternummer' => '',
                'amtsgericht' => '',
            ],
        ];

        if (null !== $consorsFinanzData && null !== $consorsFinanzData->get('birthday')) {
            /** @var RequestDataBag $birthday */
            $birthday = $consorsFinanzData->get('birthday');

            $date = (new DateTime())->setDate(
                $birthday->getInt('year'),
                $birthday->getInt('month'),
                $birthday->getInt('day')
            )->format('Y-m-d');
        } else {
            $date = $context->getCustomer()->getBirthday()->format('Y-m-d');
        }

        $personalData['person']['geburtsdatum'] = $date;

        if (SettingsTable::REQUEST_TYPE_DEBIT == $anfrageart) {
            $personalData['person']['bankverbindungen'] = [
                'iban' => $consorsFinanzData->get('bankData')->get('iban'),
                'bic' => $consorsFinanzData->get('bankData')->get('bic'),
            ];
        }

        return array_filter($personalData);
    }

    private function getCustomerSalutation(CustomerAddressEntity $addressEntity, Context $context): SalutationEntity
    {
        $criteria = new Criteria([$addressEntity->getSalutationId()]);

        /** @var null|SalutationEntity $language */
        $salutation = $this->salutationRepository->search($criteria, $context)->first();

        if (null === $salutation) {
            throw new RuntimeException('missing order customer salutation');
        }

        return $salutation;
    }

    private function getCustomerCountry(CustomerAddressEntity $addressEntity, Context $context): CountryEntity
    {
        $criteria = new Criteria([$addressEntity->getCountryId()]);

        /** @var null|CountryEntity $country */
        $country = $this->countryRepository->search($criteria, $context)->first();

        if (null === $country) {
            throw new RuntimeException('missing order country entity');
        }

        return $country;
    }

    private function getCustomerLanguage(SalesChannelContext $context): LanguageEntity
    {
        if (null === $context->getCustomer()) {
            throw new RuntimeException('missing customer');
        }

        $criteria = new Criteria([$context->getCustomer()->getLanguageId()]);
        $criteria->addAssociation('locale');

        /** @var null|LanguageEntity $language */
        $language = $this->languageRepository->search($criteria, $context->getContext())->first();

        if (null === $language) {
            throw new RuntimeException('missing customer language');
        }

        return $language;
    }
}
