<?php
namespace Mille\PlatformConsorsFinanzRK\Components\Consors\Services;

use Mille\PlatformConsorsFinanzRK\Components\ConfigReader\ConfigReaderInterface;
use Mille\PlatformConsorsFinanzRK\Components\Logger\LoggerService;
use Mille\PlatformConsorsFinanzRK\PaymentHandler\ConsorsFinanzDebitPaymentHandler;
use Mille\PlatformConsorsFinanzRK\PaymentHandler\ConsorsFinanzInvoicePaymentHandler;
use Mille\PlatformConsorsFinanzRK\Struct\Configuration;
use Shopware\Core\Checkout\Payment\PaymentMethodEntity;
use Shopware\Core\Framework\Api\Context\SystemSource;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Criteria;
use Shopware\Core\Framework\DataAbstractionLayer\Search\Filter\EqualsFilter;

class DataModel
{
    /**
     * @var LoggerService
     */
    private $loggerService;
    /**
     * @var ConfigReaderInterface
     */
    private $configReader;

    /** @var Configuration */
    private $configuration;
    /**
     * @var string
     */
    protected $salesChannelId;
    /**
     * @var EntityRepositoryInterface
     */
    private $paymentMethodRepository;

    /**
     * DataModel constructor.
     *
     * @param                               $salesChannelId
     * @param ConfigReaderInterface         $configReader
     * @param EntityRepositoryInterface     $paymentMethodRepository
     * @param LoggerService                 $loggerService
     */
    public function __construct(
        $salesChannelId,
        ConfigReaderInterface $configReader,
        EntityRepositoryInterface $paymentMethodRepository,
        LoggerService $loggerService
    ) {
        $this->configReader = $configReader;
        $this->loggerService = $loggerService;
        $this->paymentMethodRepository = $paymentMethodRepository;
        $this->refreshDependencies($salesChannelId);
    }

    /**
     * @return bool
     */
    public function isActive()
    {
        $active = (bool) $this->configuration->get('activate');
        if (!$active) {
            return false;
        } else {
            $invoice = $this->getInvoicePaymentMethod();
            $debit = $this->getDebitPaymentMethod();

            if (!$invoice->getActive() && !$debit->getActive()) {
                return false;
            } else {
                return true;
            }
        }
    }

    private function getDebitPaymentMethod()
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('handlerIdentifier', ConsorsFinanzDebitPaymentHandler::class));

        /** @var PaymentMethodEntity $debitPayment */
        $debitPayment = $this->paymentMethodRepository
            ->search($criteria, $this->getContext())
            ->first();

        return $debitPayment ?? null;
    }

    private function getInvoicePaymentMethod()
    {
        $criteria = new Criteria();
        $criteria->addFilter(new EqualsFilter('handlerIdentifier', ConsorsFinanzInvoicePaymentHandler::class));

        /** @var PaymentMethodEntity $invoicePayment */
        $invoicePayment = $this->paymentMethodRepository
            ->search($criteria, $this->getContext())
            ->first();

        return $invoicePayment ?? null;
    }

    public function getContext()
    {
        return new Context(new SystemSource());
    }

    public function log($message, $context)
    {
        $this->loggerService->error($message, $context);
    }

    public function get($key)
    {
        return $this->configuration->get($key);
    }

    public function getSettings()
    {
        $keys = [
            'activate',
            'transactionMode',
            'codeNumber',
            'password',
            'maxBasketValue',
            'conditionOrderState',
            'managementSystem',
            'conditionOrderStatus',
        ];

        $settings = [];
        foreach ($keys as $key) {
            $settings[$key] = $this->configuration->get($key);
        }

        return $settings;
    }

    /**
     * @param string $salesChannelId
     */
    public function refreshDependencies($salesChannelId = '')
    {
        $this->configuration = $this->configReader->read($salesChannelId);
        $this->salesChannelId = $salesChannelId;
    }
}
