<?php declare(strict_types=1);
namespace Mille\PlatformConsorsFinanzRK\EventListener;

use Mille\PlatformConsorsFinanzRK\Bootstrap\CustomFieldInstaller;
use Mille\PlatformConsorsFinanzRK\Components\ConfigReader\ConfigReaderInterface;
use Mille\PlatformConsorsFinanzRK\Components\SettingsTable;
use Mille\PlatformConsorsFinanzRK\PaymentMethod\ConsorsFinanzDebit;
use Mille\PlatformConsorsFinanzRK\PaymentMethod\ConsorsFinanzInvoice;
use Mille\PlatformConsorsFinanzRK\Struct\Configuration;
use Shopware\Core\Checkout\Payment\PaymentMethodCollection;
use Shopware\Core\Checkout\Payment\PaymentMethodEntity;
use Shopware\Core\Framework\Struct\ArrayStruct;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Storefront\Page\Account\Order\AccountEditOrderPageLoadedEvent;
use Shopware\Storefront\Page\Account\PaymentMethod\AccountPaymentMethodPageLoadedEvent;
use Shopware\Storefront\Page\Checkout\Confirm\CheckoutConfirmPageLoadedEvent;
use Shopware\Storefront\Page\PageLoadedEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class CheckoutConfirmPaymentMethodsEventListener implements EventSubscriberInterface
{
    /** @var Configuration */
    private $configuration;
    /**
     * @var ConfigReaderInterface
     */
    private $configReader;

    const LEGAL_AGE = 18;

    public const PAYMENT_METHODS = [
        ConsorsFinanzInvoice::class,
        ConsorsFinanzDebit::class,
    ];

    public function __construct(ConfigReaderInterface $configReader)
    {
        $this->configReader = $configReader;
    }

    public static function getSubscribedEvents(): array
    {
        return [
            CheckoutConfirmPageLoadedEvent::class      => 'hideConsorsPaymentMethods',
            AccountPaymentMethodPageLoadedEvent::class => 'hideConsorsPaymentMethods',
            AccountEditOrderPageLoadedEvent::class     => 'hideConsorsPaymentMethods',
        ];
    }

    public function hideConsorsPaymentMethods(PageLoadedEvent $event): void
    {
        $page = $event->getPage();

        if (!method_exists($page, 'getPaymentMethods') ||
            !method_exists($page, 'setPaymentMethods')) {
            return;
        }

        $this->configuration = $this->configReader->read($event->getSalesChannelContext()->getSalesChannel()->getId());

        $maxBasketValue = $this->configuration->get('maxBasketValue');
        $totalPrice = $page->getVars()['cart']->getPrice()->getTotalPrice();

        if ($totalPrice <= $maxBasketValue) {
            return;
        }

        if ($page->hasExtension(SettingsTable::EXTENSION_NAME)) {
            $extension = $page->getExtension(SettingsTable::EXTENSION_NAME);
        } else {
            $extension = $this->buildPaymentDataExtension($event->getSalesChannelContext());
        }

        $birthday = $extension['birthday']->format('Y-m-d');
        if ('' == $birthday || $this->validateBirthDay($birthday)) {
            return;
        }

        $paymentMethods = $page->getPaymentMethods();

        foreach ($this->getPaymentMethods() as $paymentMethod) {
            $paymentMethods = $this->removePaymentMethod($paymentMethods, $paymentMethod->getId());
        }

        $page->setPaymentMethods($paymentMethods);
    }

    private function getTemplateFromPaymentMethod(PaymentMethodEntity $paymentMethod): ?string
    {
        $customFields = $paymentMethod->getCustomFields();

        if (!empty($customFields[CustomFieldInstaller::TEMPLATE])) {
            return $customFields[CustomFieldInstaller::TEMPLATE];
        }

        return null;
    }

    private function buildPaymentDataExtension(SalesChannelContext $salesChannelContext): ArrayStruct
    {
        $paymentMethod = $salesChannelContext->getPaymentMethod();
        $customer = $salesChannelContext->getCustomer();

        if ($customer) {
            $customerBirthday = $customer->getBirthday();
            $customerBillingAddress = $customer->getActiveBillingAddress();
            if ($customerBillingAddress) {
                $customerPhoneNumber = $customerBillingAddress->getPhoneNumber();
            }
        }

        $template = $this->getTemplateFromPaymentMethod($paymentMethod);

        $extension = new ArrayStruct();
        $extension->set('birthday', $customerBirthday ?? null);
        $extension->set('phoneNumber', $customerPhoneNumber ?? null);
        $extension->set('template', $template ?? null);
        $extension->set('birthdayType', $this->configuration->get('birthdayType') ?? null);

        return $extension;
    }

    private function removePaymentMethod(PaymentMethodCollection $paymentMethods, string $paymentMethodId): PaymentMethodCollection
    {
        return $paymentMethods->filter(
            static function (PaymentMethodEntity $paymentMethod) use ($paymentMethodId) {
                return $paymentMethod->getId() !== $paymentMethodId;
            }
        );
    }

    private function getPaymentMethods(): array
    {
        $paymentMethods = [];

        foreach (self::PAYMENT_METHODS as $paymentMethod) {
            $paymentMethods[] = new $paymentMethod();
        }

        return $paymentMethods;
    }

    private function validateBirthDay($birthday, $age = self::LEGAL_AGE)
    {
        if (is_string($birthday)) {
            $birthday = strtotime($birthday);
        }

        if (time() - $birthday < $age * 31536000) {
            return false;
        }

        return true;
    }

    function dumb($data){
        highlight_string("<?php\n " . var_export($data, true) . "?>");
        echo '<script>document.getElementsByTagName("code")[0].getElementsByTagName("span")[1].remove() ;document.getElementsByTagName("code")[0].getElementsByTagName("span")[document.getElementsByTagName("code")[0].getElementsByTagName("span").length - 1].remove() ; </script>';
        die();
    }
}
