<?php declare(strict_types=1);
namespace Mille\PlatformConsorsFinanzRK\EventListener;

use Mille\PlatformConsorsFinanzRK\Bootstrap\CustomFieldInstaller;
use Mille\PlatformConsorsFinanzRK\Components\ConfigReader\ConfigReaderInterface;
use Mille\PlatformConsorsFinanzRK\Components\Factory\ModelFactoryInterface;
use Mille\PlatformConsorsFinanzRK\Components\SettingsTable;
use Mille\PlatformConsorsFinanzRK\PaymentMethod\ConsorsFinanzDebit;
use Mille\PlatformConsorsFinanzRK\PaymentMethod\ConsorsFinanzInvoice;
use Shopware\Core\Checkout\Payment\PaymentMethodCollection;
use Shopware\Core\Checkout\Payment\PaymentMethodEntity;
use Shopware\Core\Framework\Struct\ArrayStruct;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Storefront\Page\Account\Order\AccountEditOrderPageLoadedEvent;
use Shopware\Storefront\Page\Checkout\Confirm\CheckoutConfirmPageLoadedEvent;
use Shopware\Storefront\Page\PageLoadedEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class CheckoutConfirmTemplateEventListener implements EventSubscriberInterface
{
    /**
     * @var ConfigReaderInterface
     */
    private $configReader;

    const LEGAL_AGE = 18;

    public const PAYMENT_METHODS = [
        ConsorsFinanzInvoice::class,
        ConsorsFinanzDebit::class,
    ];
    /**
     * @var ModelFactoryInterface
     */
    private $modelFactory;

    /**
     * CheckoutConfirmTemplateEventListener constructor.
     * @param ConfigReaderInterface $configReader
     * @param ModelFactoryInterface $modelFactory
     */
    public function __construct(ConfigReaderInterface $configReader, ModelFactoryInterface $modelFactory)
    {
        $this->configReader = $configReader;
        $this->modelFactory = $modelFactory;
    }

    public static function getSubscribedEvents(): array
    {
        return [
            CheckoutConfirmPageLoadedEvent::class  => 'addConsorsPageData',
            AccountEditOrderPageLoadedEvent::class  => 'addConsorsPageData',
        ];
    }

    public function addConsorsPageData(PageLoadedEvent $event): void
    {
        $page    = $event->getPage();
        $context = $event->getSalesChannelContext();

        $model = $this->modelFactory->get($event->getSalesChannelContext()->getSalesChannel()->getId());

        if (!$model->isActive()) {
            return;
        }

        if ($page->hasExtension(SettingsTable::EXTENSION_NAME)) {
            $extension = $page->getExtension(SettingsTable::EXTENSION_NAME);
        } else {
            $extension = $this->buildPaymentDataExtension($event->getSalesChannelContext());
        }

        if (isset($page->getVars()['cart'])) {
            $totalPrice = $page->getVars()['cart']->getPrice()->getTotalPrice();
        } else {
            $totalPrice = $page->getVars()['order']->getPrice()->getTotalPrice();
        }

        $birthday = ($extension && $extension['birthday']) ? $extension['birthday']->format('Y-m-d') : '';

        //$this->dumb($event->getPage()->getPaymentMethods());
//        die('Billing = '.$event->getSalesChannelContext()->getCustomer()->getDefaultBillingAddressId(). ' and shipping = '.$event->getSalesChannelContext()->getCustomer()->getDefaultShippingAddressId());
//        $this->dumb($event->getSalesChannelContext()->getCustomer()->getDefaultShippingAddress());

        if ($totalPrice > $model->get('maxBasketValue') ||
            ($extension && '' !== $birthday && !$this->validateBirthDay($birthday)) ||
            $event->getSalesChannelContext()->getCustomer()->getDefaultBillingAddressId() !== $event->getSalesChannelContext()->getCustomer()->getDefaultShippingAddressId()
        ) {
            $this->hideConsorsPaymentMethods($event);
            return;
        }

        if ($this->isConsorsPayment($context->getPaymentMethod())) {
            $page->addExtension(SettingsTable::EXTENSION_NAME, $extension);
        }
    }

    public function hideConsorsPaymentMethods(PageLoadedEvent $event): void
    {
        $page = $event->getPage();

        if (!method_exists($page, 'getPaymentMethods') ||
            !method_exists($page, 'setPaymentMethods')) {
            return;
        }

        $paymentMethods = $page->getPaymentMethods();

        foreach ($this->getPaymentMethods() as $paymentMethod) {
            $paymentMethods = $this->removePaymentMethod($paymentMethods, $paymentMethod->getId());
        }

        $page->setPaymentMethods($paymentMethods);
    }

    private function isConsorsPayment(PaymentMethodEntity $paymentMethod): bool
    {
        $customFields = $paymentMethod->getCustomFields();

        if (empty($customFields[CustomFieldInstaller::IS_CONSORS_FINANZ]) ||
            !$customFields[CustomFieldInstaller::IS_CONSORS_FINANZ] ||
            !$paymentMethod->getActive()
        ) {
            return false;
        }

        return true;
    }

    private function buildPaymentDataExtension(SalesChannelContext $salesChannelContext): ArrayStruct
    {
        $paymentMethod = $salesChannelContext->getPaymentMethod();
        $customer = $salesChannelContext->getCustomer();

        if ($customer) {
            $customerBirthday = $customer->getBirthday();
            $customerBillingAddress = $customer->getActiveBillingAddress();
            if ($customerBillingAddress) {
                $customerPhoneNumber = $customerBillingAddress->getPhoneNumber();
            }
        }

        $template = $this->getTemplateFromPaymentMethod($paymentMethod);

        $extension = new ArrayStruct();
        $extension->set('birthday', $customerBirthday ?? null);
        $extension->set('phoneNumber', $customerPhoneNumber ?? null);
        $extension->set('template', $template ?? null);

        return $extension;
    }

    private function getTemplateFromPaymentMethod(PaymentMethodEntity $paymentMethod): ?string
    {
        $customFields = $paymentMethod->getCustomFields();

        if (!empty($customFields[CustomFieldInstaller::TEMPLATE])) {
            return $customFields[CustomFieldInstaller::TEMPLATE];
        }

        return null;
    }

    private function removePaymentMethod(PaymentMethodCollection $paymentMethods, string $paymentMethodId): PaymentMethodCollection
    {
        return $paymentMethods->filter(
            static function (PaymentMethodEntity $paymentMethod) use ($paymentMethodId) {
                return $paymentMethod->getId() !== $paymentMethodId;
            }
        );
    }

    private function getPaymentMethods(): array
    {
        $paymentMethods = [];

        foreach (self::PAYMENT_METHODS as $paymentMethod) {
            $paymentMethods[] = new $paymentMethod();
        }

        return $paymentMethods;
    }

    private function validateBirthDay($birthday, $age = self::LEGAL_AGE)
    {
        if (is_string($birthday)) {
            $birthday = strtotime($birthday);
        }

        if (time() - $birthday < $age * 31536000) {
            return false;
        }

        return true;
    }

    function dumb($data){
        highlight_string("<?php\n " . var_export($data, true) . "?>");
        echo '<script>document.getElementsByTagName("code")[0].getElementsByTagName("span")[1].remove() ;document.getElementsByTagName("code")[0].getElementsByTagName("span")[document.getElementsByTagName("code")[0].getElementsByTagName("span").length - 1].remove() ; </script>';
        die();
    }
}
