<?php declare(strict_types=1);
namespace Mille\PlatformConsorsFinanzRK\EventListener;

use DateTime;
use Mille\PlatformConsorsFinanzRK\PaymentHandler\Event\BeforePaymentEvent;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class UserDataEventListener implements EventSubscriberInterface
{
    /**
     * @var EntityRepositoryInterface
     */
    private $addressRepository;

    /**
     * @var EntityRepositoryInterface
     */
    private $customerRepository;

    /**
     * @var EntityRepositoryInterface
     */
    private $orderAddressRepository;

    public function __construct(
        EntityRepositoryInterface $customerRepository,
        EntityRepositoryInterface $orderAddressRepository,
        EntityRepositoryInterface $addressRepository
    ) {
        $this->customerRepository = $customerRepository;
        $this->orderAddressRepository = $orderAddressRepository;
        $this->addressRepository = $addressRepository;
    }

    public static function getSubscribedEvents()
    {
        return [
            BeforePaymentEvent::class => 'saveUserData',
        ];
    }

    public function saveUserData(BeforePaymentEvent $event)
    {
        $paymentRequestData = $event->getPaymentRequestData();
        $order = $paymentRequestData->getOrder();
        $customer = $order->getOrderCustomer()->getCustomer();
        $orderBillingAddress = $order->getAddresses()->get($order->getBillingAddressId());
        $dataBag = $paymentRequestData->getRequestDataBag();

        $consorsFinanzData = $dataBag->get('consorsfinanz');
        if (!$consorsFinanzData) {
            return;
        }

        if ($customer) {
            $customerUpdates = [];

            /** @var RequestDataBag $consorsFinanzData */
            if ($consorsFinanzData->has('birthday')) {
                /** @var RequestDataBag $birthday */
                $birthday = $consorsFinanzData->get('birthday');

                $date = (new DateTime())->setDate(
                    $birthday->getInt('year'),
                    $birthday->getInt('month'),
                    $birthday->getInt('day')
                );

                $customer->setBirthday($date);
                $customerUpdates['birthday'] = $date;
            }

            if (count($customerUpdates) > 0) {
                $this->customerRepository->upsert([array_merge(
                    [
                        'id' => $customer->getId(),
                        'versionId' => $customer->getVersionId(),
                    ],
                    $customerUpdates
                )], $event->getContext());
            }
        }

        $defaultBillingAddressId = $customer->getDefaultBillingAddressId();
        if ($orderBillingAddress || $defaultBillingAddressId) {
            $billingAddressUpdates = [];

            if ($consorsFinanzData->has('phone') && !empty($phone = $consorsFinanzData->get('phone'))) {
                $orderBillingAddress->setPhoneNumber($phone);
                $billingAddressUpdates['phoneNumber'] = $phone;
            }

            if (count($billingAddressUpdates) > 0) {
                if ($orderBillingAddress) {
                    $this->orderAddressRepository->upsert([array_merge(
                        [
                            'id' => $orderBillingAddress->getId(),
                            'versionId' => $orderBillingAddress->getVersionId(),
                        ],
                        $billingAddressUpdates
                    )], $event->getContext());
                }

                if ($defaultBillingAddressId) {
                    $this->addressRepository->upsert([array_merge(
                        [
                            'id' => $defaultBillingAddressId,
                        ],
                        $billingAddressUpdates
                    )], $event->getContext());
                }
            }
        }
    }
}
