<?php declare(strict_types=1);
namespace Mille\PlatformConsorsFinanzRK\PaymentHandler;

use DateTime;
use LogicException;
use Mille\PlatformConsorsFinanzRK\Bootstrap\CustomFieldInstaller;
use Mille\PlatformConsorsFinanzRK\Components\ConfigReader\ConfigReaderInterface;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Api\ConsorsSoapClient;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Request\AuthorizeRequestFactory;
use Mille\PlatformConsorsFinanzRK\Components\Logger\LoggerService;
use Mille\PlatformConsorsFinanzRK\PaymentHandler\Constraint\AgbNotBlank;
use Mille\PlatformConsorsFinanzRK\PaymentHandler\Constraint\Birthday;
use Mille\PlatformConsorsFinanzRK\PaymentHandler\Constraint\BirthdayNotBlank;
use Mille\PlatformConsorsFinanzRK\PaymentHandler\Constraint\IsOfLegalAge;
use Mille\PlatformConsorsFinanzRK\PaymentHandler\Constraint\PhoneNotBlank;
use Mille\PlatformConsorsFinanzRK\Struct\Configuration;
use Mille\PlatformConsorsFinanzRK\Struct\PaymentTransaction;
use Mille\PlatformConsorsFinanzRK\Util\CriteriaHelper;
use Shopware\Core\Checkout\Order\OrderEntity;
use Shopware\Core\Checkout\Payment\Cart\PaymentHandler\SynchronousPaymentHandlerInterface;
use Shopware\Core\Framework\Context;
use Shopware\Core\Framework\DataAbstractionLayer\EntityRepositoryInterface;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Contracts\Translation\TranslatorInterface;

abstract class AbstractPaymentHandler  implements SynchronousPaymentHandlerInterface
{
    /** @var TranslatorInterface */
    protected $translator;
    /**
     * @var ConsorsSoapClient
     */
    protected $client;
    /**
     * @var AuthorizeRequestFactory
     */
    protected $authRequestFactory;
    /**
     * @var ContainerInterface
     */
    protected $container;

    public const ERROR_SNIPPET_VIOLATION_PREFIX = 'error.VIOLATION::';
    /**
     * @var RequestStack
     */
    protected $requestStack;
    /**
     * @var ConfigReaderInterface
     */
    protected $configReader;

    protected $session;
    /**
     * @var EntityRepositoryInterface
     */
    protected $customerRepository;
    /**
     * @var EntityRepositoryInterface
     */
    protected $orderAddressRepository;
    /**
     * @var EntityRepositoryInterface
     */
    protected $addressRepository;
    /**
     * @var EntityRepositoryInterface
     */
    protected $orderRepository;
    /**
     * @var LoggerService
     */
    protected $loggerService;

    /** @var Configuration */
    protected $configuration;

    public function __construct(
        ConfigReaderInterface $configReader,
        TranslatorInterface $translator,
        RequestStack $requestStack,
        AuthorizeRequestFactory $authRequestFactory,
        ConsorsSoapClient $client,
        ContainerInterface $container,
        EntityRepositoryInterface $orderRepository,
        EntityRepositoryInterface $customerRepository,
        EntityRepositoryInterface $orderAddressRepository,
        EntityRepositoryInterface $addressRepository,
        LoggerService $loggerService
    ) {
        $this->authRequestFactory    = $authRequestFactory;
        $this->translator            = $translator;
        $this->client = $client;
        $this->container = $container;
        $this->requestStack = $requestStack;
        $this->configReader = $configReader;
        $this->customerRepository = $customerRepository;
        $this->orderAddressRepository = $orderAddressRepository;
        $this->addressRepository = $addressRepository;
        $this->orderRepository = $orderRepository;
        $this->session = $container->get('session');
        $this->loggerService = $loggerService;
    }

    protected function fetchRequestData(): RequestDataBag
    {
        $request = $this->requestStack->getCurrentRequest();

        if (null === $request) {
            throw new LogicException('missing current request');
        }

        return new RequestDataBag($request->request->all());
    }

    /**
     * @return OrderEntity|null
     */
    protected function getOrderWithAssociations(OrderEntity $order, Context $context): OrderEntity
    {
        return $this->orderRepository->search(CriteriaHelper::getCriteriaForOrder($order->getId()), $context)->first();
    }

    public function getValidationDefinitions(Request $request, SalesChannelContext $salesChannelContext): array
    {
        $this->configuration = $this->configReader->read($salesChannelContext->getSalesChannel()->getId());
        $consorsFinanzData = $request->get('consorsfinanz');

        $customer = $salesChannelContext->getCustomer();

        $customerBirthday = $customerPhoneNumber = null;
        if ($customer) {
            $customerBirthday = $customer->getBirthday();
            $customerBillingAddress = $customer->getActiveBillingAddress();
            if ($customerBillingAddress) {
                $customerPhoneNumber = $customerBillingAddress->getPhoneNumber();
            }
        }

        $validations['agb'] = [new AgbNotBlank()];

        if (null == $customerPhoneNumber) {
            $validations['phone'] = [new PhoneNotBlank()];
        }

        if (null === $customerBirthday) {
            if (isset($consorsFinanzData['birthday'])) {
                $validations['birthday'] = [
                    new BirthdayNotBlank(),
                    new Birthday(['message' => self::ERROR_SNIPPET_VIOLATION_PREFIX . Birthday::ERROR_NAME]),
                    new IsOfLegalAge(['message' => self::ERROR_SNIPPET_VIOLATION_PREFIX . IsOfLegalAge::TOO_YOUNG_ERROR_NAME]),
                ];
            }
        }

        return $validations;
    }

    public function saveUserData(
        RequestDataBag $dataBag,
        PaymentTransaction $transaction,
        SalesChannelContext $salesChannelContext
    ) {
        $order = $this->getOrderWithAssociations($transaction->getOrder(), $salesChannelContext->getContext());
        $customer = $order->getOrderCustomer()->getCustomer();
        $orderBillingAddress = $order->getAddresses()->get($order->getBillingAddressId());

        $this->configuration = $this->configReader->read($salesChannelContext->getSalesChannel()->getId());

        $consorsFinanzData = $dataBag->get('consorsfinanz');
        if (!$consorsFinanzData) {
            return;
        }

        if ($customer) {
            $customerUpdates = [];

            /** @var RequestDataBag $consorsFinanzData */
            if ($consorsFinanzData->has('birthday')) {
                /** @var RequestDataBag $birthday */
                $birthday = $consorsFinanzData->get('birthday');

                $date = (new DateTime())->setDate(
                    $birthday->getInt('year'),
                    $birthday->getInt('month'),
                    $birthday->getInt('day')
                );;
                $customer->setBirthday($date);
                $customerUpdates['birthday'] = $date;
            }

            if (count($customerUpdates) > 0) {
                $this->customerRepository->upsert([array_merge(
                    [
                        'id' => $customer->getId(),
                        'versionId' => $customer->getVersionId(),
                    ],
                    $customerUpdates
                )], $salesChannelContext->getContext());
            }

            /** @var RequestDataBag $bankData */
            $bankData = $consorsFinanzData->get('bankData');
            if (!empty($bankData)) {
                $this->customerRepository->upsert([
                    [
                        'id' => $customer->getId(),
                        'customFields' => [
                            CustomFieldInstaller::CONSORS_FINANZ_USER_IBAN => $bankData->get('iban'),
                            CustomFieldInstaller::CONSORS_FINANZ_USER_BIC => $bankData->get('bic'),
                        ],
                    ],
                ], $salesChannelContext->getContext());
            }
        }

        $defaultBillingAddressId = $customer->getDefaultBillingAddressId();
        if ($orderBillingAddress || $defaultBillingAddressId) {
            $billingAddressUpdates = [];

            if ($consorsFinanzData->has('phone') && !empty($phone = $consorsFinanzData->get('phone'))) {
                $orderBillingAddress->setPhoneNumber($phone);
                $billingAddressUpdates['phoneNumber'] = $phone;
            }

            if (count($billingAddressUpdates) > 0) {
                if ($orderBillingAddress) {
                    $this->orderAddressRepository->upsert([array_merge(
                        [
                            'id' => $orderBillingAddress->getId(),
                            'versionId' => $orderBillingAddress->getVersionId(),
                        ],
                        $billingAddressUpdates
                    )], $salesChannelContext->getContext());
                }

                if ($defaultBillingAddressId) {
                    $this->addressRepository->upsert([array_merge(
                        [
                            'id' => $defaultBillingAddressId,
                        ],
                        $billingAddressUpdates
                    )], $salesChannelContext->getContext());
                }
            }
        }
    }

    function dumb($data){
        highlight_string("<?php\n " . var_export($data, true) . "?>");
        echo '<script>document.getElementsByTagName("code")[0].getElementsByTagName("span")[1].remove() ;document.getElementsByTagName("code")[0].getElementsByTagName("span")[document.getElementsByTagName("code")[0].getElementsByTagName("span").length - 1].remove() ; </script>';
        die();
    }
}
