<?php declare(strict_types=1);

namespace Mille\PlatformConsorsFinanzRK\PaymentHandler;

use Mille\PlatformConsorsFinanzRK\Components\Consors\Exception\ConsorsFinanzRequestException;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Response\BoniCheck2Response;
use Mille\PlatformConsorsFinanzRK\Components\Consors\Response\Order2Response;
use Mille\PlatformConsorsFinanzRK\Components\SettingsTable;
use Mille\PlatformConsorsFinanzRK\Struct\PaymentTransaction;
use Shopware\Core\Checkout\Payment\Cart\SyncPaymentTransactionStruct;
use Shopware\Core\Checkout\Payment\Exception\SyncPaymentProcessException;
use Shopware\Core\Framework\Validation\DataBag\RequestDataBag;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Component\HttpFoundation\Request;
use Throwable;

class ConsorsFinanzDebitPaymentHandler extends AbstractPaymentHandler
{
    use DebitValidationTrait;

    const REQUEST_TYPE = SettingsTable::REQUEST_TYPE_DEBIT;

    /**
     * {@inheritdoc}
     */
    public function pay(SyncPaymentTransactionStruct $transaction, RequestDataBag $dataBag, SalesChannelContext $salesChannelContext): void
    {
        $paymentTransaction = PaymentTransaction::fromSyncPaymentTransactionStruct($transaction, $transaction->getOrder());

        $this->saveUserData($dataBag, $paymentTransaction, $salesChannelContext);

        $boniCheck2Request = $this->authRequestFactory->getRequestParameters(
            $paymentTransaction,
            $dataBag,
            $salesChannelContext,
            SettingsTable::REQUEST_TYPE_DEBIT,
            'boni2'
        );

        try {
            // Boni2 check
            /** @var BoniCheck2Response $boniCheck2Response */
            $boniCheck2Response = $this->client->boniCheck2($boniCheck2Request);
            if (0 !== $boniCheck2Response->getFehlergrund()) {
                $this->session->getFlashBag()->add('danger', $boniCheck2Response->getFehlertext());

                throw new ConsorsFinanzRequestException(
                    $boniCheck2Response->getFehlertext()
                );
            } else {
                // Order2 check
                $order2Request = $this->authRequestFactory->getRequestParameters(
                    $paymentTransaction,
                    $dataBag,
                    $salesChannelContext,
                    SettingsTable::REQUEST_TYPE_DEBIT,
                    'order2'
                );

                /** @var Order2Response $order2Response */
                $order2Response = $this->client->order2($order2Request);

                if (0 === $order2Response->getFehlergrund()) {
                    $this->session->remove('consorsUserReference');
                } else {
                    $this->loggerService->error($order2Response->getFehlertext());
                    throw new ConsorsFinanzRequestException(
                        $order2Response->getFehlertext()
                    );
                }
            }
        } catch (ConsorsFinanzRequestException $exception) {
            $this->loggerService->error($exception->getMessage());
            throw new SyncPaymentProcessException(
                $transaction->getOrderTransaction()->getId(),
                $exception->getMessage()
            );
        } catch (Throwable $exception) {
            $this->loggerService->error($exception->getMessage());
            throw new SyncPaymentProcessException(
                $transaction->getOrderTransaction()->getId(),
                $this->translator->trans('consorsfinanz.errorMessages.genericError')
            );
        }
    }

    public function getValidationDefinitions(Request $request, SalesChannelContext $salesChannelContext): array
    {
        $validations = parent::getValidationDefinitions($request, $salesChannelContext);
        return array_merge($validations, $this->getDebitConstraints($request, $salesChannelContext));
    }
}
